#! /usr/bin/env python3

from sys import argv
from subprocess import run
import os
import multiprocessing as mp

def tblastn(db,query,outfile):
    cmd = 'tblastn -db {} -query {} -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore sseq" -evalue 1e-3 -word_size 7 -out {}'.format(db,query,outfile)
    run(cmd,shell=True)
    return(0)

def blastn(db,query,outfile):
    cmd = 'blastn -db {} -query {} -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore sseq" -evalue 1e-3 -word_size 9 -out {}'.format(db,query,outfile)
    run(cmd,shell=True)
    return(0)

def clasp(infile,outfile,l=0.5,e=0):
    cmd = './clasp.x -m -i {} -c 7 8 9 10 12 -C 1 2 -l {} -e {} -o {}'.format(infile,l,e,outfile).split()
    run(cmd)
    return(0)

def exec_blast_clasp(org):
    
    if argv[3] == 'prot':
        tblastn(f'blastdbs/{org}',f'{protein}',f'blast_out_both/{org}')
    else:
        blastn(f'blastdbs/{org}',f'{protein}',f'blast_out_both/{org}')

    
    with open(f'blast_out_both/{org}','r') as f:
        newlines_forward = []
        newlines_reverse = []
        for line in f:
            line = line.split()
            if line[0] == '#' or len(line) == 0:
                continue
            if int(line[8]) > int(line[9]):
                line[8],line[9] = line[9],line[8]
                newlines_reverse.append('\t'.join(line))
            else:
                newlines_forward.append('\t'.join(line))

    with open(f'blast_out_forward/{org}','w') as f:
        f.write('\n'.join(newlines_forward))
    with open(f'blast_out_reverse/{org}','w') as f:
        f.write('\n'.join(newlines_reverse))

    for orientation in ['forward','reverse']:
    
        clasp(f'blast_out_{orientation}/{org}',f'clasp_out_{orientation}/{org}')

    hit_no = 1
    with open(f'temp_coords_{org}','w') as out:
        for orientation in ['forward','reverse']:
            with open(f'clasp_out_{orientation}/{org}') as f:
                for line in f:
                    if line[0] == '#':
                        continue
                    line = line.split()
                    orig_prot = line[1]
                    chromo = line[2]
                    start = line[5]
                    end = line[6]
                    out.write(f'{org}\t{chromo}\t{start}\t{end}\t{orientation}\thit-{hit_no}_{org}_{orig_prot}\n')
                    hit_no += 1

    return(0)

def mkbdb(org):
    if not os.path.isfile(path + f"/blastdbs/{org}.nsq"):
        run(f'makeblastdb -dbtype nucl -in {genomes}/{org}.fasta -out blastdbs/{org}',shell=True)

if __name__ == "__main__":
    
    protein = argv[1]
    genomes = argv[2]
    path = os.getcwd()

    run('mkdir -p blastdbs',shell=True)   
    run('mkdir -p blast_out_both',shell=True)   
    run('mkdir -p blast_out_forward',shell=True)   
    run('mkdir -p blast_out_reverse',shell=True)   
    run('mkdir -p clasp_out_forward',shell=True)   
    run('mkdir -p clasp_out_reverse',shell=True)   
    orgs = []
    with open('orgs') as f:
        for line in f:
            org = line.strip()
            orgs.append(org)

    with mp.Pool(processes=11) as pool:
        res = pool.map_async(mkbdb,orgs).get()
            
    with mp.Pool(processes=11) as pool:
        res = pool.map_async(exec_blast_clasp,orgs).get()



#! /usr/bin/env python3

import sys
import re
from get_mapping import get_mapping

if __name__ == "__main__":

    org_mapping,chr_mapping = get_mapping()
    with open(sys.argv[1]) as f:
        txt = f.read()
    js = [x for x in list(reversed(sorted(chr_mapping,key=len))) if re.match(r'\d+org',x)]
    for j in js:
        ks = [x for x in list(reversed(sorted(chr_mapping[j],key=len))) if 'orgchr' in x]
        for k in ks:
            txt = re.sub(k,chr_mapping[j][k],txt)
    ks = [x for x in list(reversed(sorted(org_mapping,key=len))) if re.match(r'\d+org',x)]
    for k in ks:
        txt = re.sub(k,org_mapping[k],txt)
    with open(sys.argv[2],'w') as f:
        f.write(txt)



#! /usr/bin/env python3


import sys
sys.path.append('../../utils/')
from get_mapping import get_mapping

import pickle
from random import randint
from matplotlib.pyplot import cm
from pygenomeviz import GenomeViz
import numpy as np
from subprocess import run
from bisect import bisect_left,bisect_right

def get_gff():

    gff = {}
    with open('../MCScanX/MCScanX_plus_coords_only_relevant.gff') as f:
        for line in f:
            chromo,name,start,end = line.strip().split()
            gff[name] = (chromo,start,end)

    return gff

def get_homology(gff):

    clusters = {}
    alloc = {}
    cluster_id = 1

    homology = {}
    with open('../MCScanX/MCScanX_plus_coords_only_relevant.homology') as f:
        for line in f:
            name1,name2,score = line.strip().split()
            if 'CUSTOM' in name1 or 'CUSTOM' in name2:
                continue
            chromo1,start1,end1 = gff[name1]
            chromo2,start2,end2 = gff[name2]
            org1 = org_mapping[name1.split('ele')[0].split('chr')[0]]
            org2 = org_mapping[name2.split('ele')[0].split('chr')[0]]
            chromo1 = chr_mapping[org1][chromo1]
            chromo2 = chr_mapping[org2][chromo2]
            start1 = int(start1)
            start2 = int(start2)
            end1 = int(end1)
            end2 = int(end2)
            
            if org1 not in homology:
                homology[org1] = {}
            if chromo1 not in homology[org1]:
                homology[org1][chromo1] = {}
            if start1 not in homology[org1][chromo1]:
                homology[org1][chromo1][start1] = {'end':end1,'matches':{},'score':int(score)}
            homology[org1][chromo1][start1]['matches'][org2] = (chromo2,start2,int(score))

            if org2 not in homology:
                homology[org2] = {}
            if chromo2 not in homology[org2]:
                homology[org2][chromo2] = {}
            if start2 not in homology[org2][chromo2]:
                homology[org2][chromo2][start2] = {'end':end2,'matches':{},'score':int(score)}
            homology[org2][chromo2][start2]['matches'][org1] = (chromo1,start1,int(score))

            if org1 not in alloc:
                alloc[org1] = {}
            if chromo1 not in alloc[org1]:
                alloc[org1][chromo1] = {}
            if org2 not in alloc:
                alloc[org2] = {}
            if chromo2 not in alloc[org2]:
                alloc[org2][chromo2] = {}
            if start1 not in alloc[org1][chromo1] and start2 not in alloc[org2][chromo2]:
                clusters[cluster_id] = {}
                clusters[cluster_id][org1] = {}
                clusters[cluster_id][org2] = {}
                clusters[cluster_id][org1][chromo1] = set()
                clusters[cluster_id][org2][chromo2] = set()
                clusters[cluster_id][org1][chromo1].add(start1)
                clusters[cluster_id][org2][chromo2].add(start2)
                alloc[org1][chromo1][start1] = cluster_id
                alloc[org2][chromo2][start2] = cluster_id
                cluster_id += 1
            elif start1 not in alloc[org1][chromo1]:
                c_id = alloc[org2][chromo2][start2]
                if org1 not in clusters[c_id]:
                    clusters[c_id][org1] = {}
                if chromo1 not in clusters[c_id][org1]:
                    clusters[c_id][org1][chromo1] = set()
                clusters[c_id][org1][chromo1].add(start1)
                alloc[org1][chromo1][start1] = alloc[org2][chromo2][start2]
            elif start2 not in alloc[org2]:
                c_id = alloc[org1][chromo1][start1]
                if org2 not in clusters[c_id]:
                    clusters[c_id][org2] = {}
                if chromo2 not in clusters[c_id][org2]:
                    clusters[c_id][org2][chromo2] = set()
                clusters[c_id][org2][chromo2].add(start2)
                alloc[org2][chromo2][start2] = alloc[org1][chromo1][start1]
            elif alloc[org1][chromo1][start1] != alloc[org2][chromo2][start2]:
                to_del = alloc[org1][chromo1][start1]
                c_id = alloc[org2][chromo2][start2]
                for org11,bib in clusters[to_del].items():
                    if org11 not in clusters[c_id]:
                        clusters[c_id][org11] = {}
                    for chromo11,s in bib.items():
                        if chromo11 not in clusters[alloc[org2][start2]][org11]:
                            clusters[c_id][org11][chromo11] = set()
                        for x in s:
                            clusters[c_id][org11][chromo11].add(x)
                            alloc[org11][chromo11][x] = c_id
                del clusters[to_del]

    return homology,clusters,alloc

def main(pivot,pivot_coords):
    
    g_anchors = {}


    for p_start,p_end,p_ori,p_chromo,p_hit_name in pivot_coords:

        p_start,p_end = int(p_start),int(p_end)

        print('+++++++++++++++++++++++++')
        print(f'element of {pivot}:') 
        print(f'chromo:{p_chromo} start:{p_start} end:{p_end} ori:{p_ori} hit name: {p_hit_name}')

        if p_chromo not in sorted_homology_starts[pivot]:
            print(f'no anchors on {p_chromo}...skippping')
            continue
        lower_bound = max(p_start-margin1,0)
        if lower_bound == 0:
            bigger_idx_start = 0
        else:
            bigger_idx_start = bisect_right(sorted_homology_starts[pivot][p_chromo],lower_bound)
        upper_bound = min(p_end+margin1,homology[pivot][p_chromo][sorted_homology_starts[pivot][p_chromo][-1]]['end'])
        if upper_bound == homology[pivot][p_chromo][sorted_homology_starts[pivot][p_chromo][-1]]['end']:
            smaller_idx_end = len(sorted_homology_starts[pivot][p_chromo])
        else:
            smaller_idx_end = bisect_left(sorted_homology_starts[pivot][p_chromo],upper_bound)
        while smaller_idx_end > 0 and homology[pivot][p_chromo][sorted_homology_starts[pivot][p_chromo][smaller_idx_end-1]]['end'] > p_end + margin1:
            smaller_idx_end -= 1
        anchors = sorted_homology_starts[pivot][p_chromo][bigger_idx_start:smaller_idx_end]
        if len(anchors) == 0:
            print('there are no suitable anchors around this elements')
            continue
        if p_chromo not in g_anchors:
            g_anchors[p_chromo] = {}
        if p_start not in g_anchors[p_chromo]:
            g_anchors[p_chromo][p_start] = set()
        g_anchors[p_chromo][p_start].update(anchors)

    return(g_anchors)

if __name__ == "__main__":
    
    margin1 = int(sys.argv[1])
    if len(sys.argv) > 2:
        margin2 = int(sys.argv[2])
    else:
        margin2 = margin1
  
    org_mapping,chr_mapping = get_mapping()

    sorted_starts = {}
    coords_bib = {}
    with open('../coords') as f:
        for line in f:
            line = line.strip().split()
            org,chromo,start,end,ori,hit_name = line
            start = int(start)
            end = int(end)
            if org not in coords_bib:
                coords_bib[org] = {}
                sorted_starts[org] = {}
            if chromo not in coords_bib[org]:
                coords_bib[org][chromo] = {}
                sorted_starts[org][chromo] = []
            coords_bib[org][chromo][start] = (start,end,ori,chromo,hit_name)
            sorted_starts[org][chromo].append(start)
    for org,bib in sorted_starts.items():
        for chromo,starts in bib.items():
            sorted_starts[org][chromo] = sorted(starts)

    gff = get_gff()
    homology,clusters,alloc = get_homology(gff)

    sorted_homology_starts = {}
    
    anchors = {}

    for pivot in coords_bib:
        sorted_homology_starts[pivot] = {}
        if pivot not in homology:
            continue
        for chromo,starts in homology[pivot].items():
            sorted_homology_starts[pivot][chromo] = sorted(list(starts.keys()))
        pivot_coords = []
        for chromo,starts in coords_bib[pivot].items():
            pivot_coords += list(starts.values())
        anchors[pivot] = main(pivot,pivot_coords)


    tol = 10000
    check = {}
    drawing_order = {}
    cluster_alloc = {}
    order_c = 1
    ele_draw_alloc = {}
    for org,bib in anchors.items():
        if org not in ele_draw_alloc:
            ele_draw_alloc[org] = {}
        for chromo,bib2 in bib.items():
            for ele_start,ass in bib2.items():
                for i in ass:
                    if alloc[org][chromo][i] not in check:
                        check[alloc[org][chromo][i]] = set()
                    if alloc[org][chromo][i] not in cluster_alloc:
                        cluster_alloc[alloc[org][chromo][i]] = set()
                    check[alloc[org][chromo][i]].add(i)
                    cluster_alloc[alloc[org][chromo][i]].add((org,ele_start))
    for c_id,members in cluster_alloc.items():
        order_c_alt = set()
        drawing_order[order_c] = set()
        for org,ele_start in members:
            if ele_start in ele_draw_alloc[org]:
                order_c_alt.add(ele_draw_alloc[org][ele_start])
        for o_c in order_c_alt:
            for org,ele_start in drawing_order[o_c]:
                ele_draw_alloc[org][ele_start] = order_c
                drawing_order[order_c].add((org,ele_start))
            del drawing_order[o_c]
        for org,ele_start in members:
            ele_draw_alloc[org][ele_start] = order_c
            drawing_order[order_c].add((org,ele_start))
        order_c += 1

    final_drawing_order = {}
    c_new = 1
    for c,members in drawing_order.items():
        if len(members) > 1:
            final_drawing_order[c_new] = members
            c_new += 1

    counter = {}
    no_colors = 0
    for c_id,members in check.items():
        alloc_times = len(members)
        if alloc_times not in counter:
            counter[alloc_times] = 0
        counter[alloc_times] += 1
        if alloc_times > 1:
            clusters[c_id]['color'] = 1
            no_colors += 1
    
    color = cm.rainbow(np.linspace(0, 1, no_colors))
    used = {}
    for c,bib in clusters.items():
        if 'color' in bib:
            ri = randint(0,len(color)-1)
            while ri in used:
                ri = randint(0,len(color)-1)
            used[ri] = True
            bib['color'] = color[ri]
    
    run(f'rm -rf clusters_from_draw_simple && mkdir -p clusters_from_draw_simple && mkdir -p clusters_from_draw_simple/{margin1}',shell=True)
    run(f'rm -rf images_draw_simple && mkdir -p images_draw_simple && mkdir -p images_draw_simple/{margin1}',shell=True)

    no_rm_ones_because_all_tracks_same_species = 0
    for c,members in final_drawing_order.items():
        c -= no_rm_ones_because_all_tracks_same_species
        gv = GenomeViz()
        out = open(f'clusters_from_draw_simple/{margin1}/cluster_{c}.txt','w')
        out.write(f'cluster {c}:')
        names = set()
        for org,ele_start in members:
            out.write('--------------------\n')
            out.write(f'species {org}\n')
            min_dis_to_gene = 1e15
            for chromo,bib2 in anchors[org].items():
                if ele_start not in bib2:
                    continue
                ass = bib2[ele_start]
                to_draw = []
                mini = 1e15
                maxi = 0

                start,end,ori,chro,hit_name = coords_bib[org][chromo][ele_start]
                start_gene = start
                end_gene = end
                if ori == 'forward':
                    ori = 1
                    ori_orig = 1
                else:
                    ori = -1
                    ori_orig = -1
                if ori == -1:
                    turn = True
                else:
                    turn = False
                ori = 1
                to_draw.append([start, end,'red','arrow',hit_name,0,"center","center",ori])
                out.write(f'element on chromosome {chromo} start {start} end {end} strand {ori_orig}\n')
                if start < mini:
                    mini = start
                if end > maxi:
                    maxi = end

                for i in ass:
                    start = i
                    if len(check[alloc[org][chromo][start]]) < 2:
                        continue
                    end = homology[org][chromo][start]['end']
                    score = homology[org][chromo][start]['score']
                    mid = start + (end-start)/2
                    start_score = mid - score/2
                    end_score = mid + score/2
                    if start_score >= start_gene and start_score <= end_gene:
                        continue
                    if end_score >= start_gene and end_score <= end_gene:
                        continue
                    if end_score < start_gene:
                        dis_to_gene = start_gene - end_score
                        start_score = (-1,dis_to_gene)
                    elif start_score > end_gene:
                        dis_to_gene = start_score - end_gene
                        start_score = (1,dis_to_gene)
                    else:
                        continue
                    if dis_to_gene < 1000:
                        continue

                    if dis_to_gene < min_dis_to_gene:
                        min_dis_to_gene = dis_to_gene

                    start_score = start_score 
                    to_draw.append([start_score[0], start_score[1], alloc[org][chromo][start],'bigbox',score])#,alloc[org][chromo][i]])
                    if start < mini:
                        mini = start
                    if end > maxi:
                        maxi = end
                name, genome_size = org + '_' + str(chromo) + '_' + str(ele_start), 2*margin1 + tol
                names.add(name)
                track = gv.add_feature_track(name,genome_size)
                out.write(f'has {len(to_draw)-1} anchors (one or more significant alignments) that are shared with something in this cluster\n')
                for ele in to_draw:
                    if len(ele) > 5:
                        ele[0] = int(genome_size/2 - end_gene + start_gene)
                        ele[1] = int(genome_size/2 + end_gene - start_gene)
                    else:
                        if ele[0] == -1:

                            new_end = genome_size/2 - end_gene + start_gene - ele[1]
                            new_start = new_end - ele[4]

                        else:

                            new_start = genome_size/2 - end_gene + start_gene + ele[1]
                            new_end = new_start + ele[4]
                        ele[0] = int(new_start)
                        ele[1] = int(new_end)

                        if turn:
                            temp = genome_size - ele[0]
                            ele[0] = int(genome_size - ele[1])
                            ele[1] = int(temp)

                    if len(ele) > 5:
                        track.add_feature(ele[0],ele[1],facecolor=ele[2],plotstyle=ele[3],label=ele[4],text_kws=dict(rotation=ele[5],vpos=ele[6],hpos=ele[7]),strand=ele[8])
                    else:
                        track.add_feature(ele[0],ele[1],facecolor=clusters[ele[2]]['color'],plotstyle=ele[3],label='')
            out.write('--------------------\n')
        if len(names) == 1:
            out.close()
            run(f'rm clusters_from_draw_simple/{margin1}/cluster_{c}.txt',shell=True)
            no_rm_ones_because_all_tracks_same_species += 1
            continue
        gv.savefig(f'images_draw_simple/{margin1}/cluster_{c}.svg')
        gv.savefig(f'images_draw_simple/{margin1}/cluster_{c}.png')
        out.close()



#! /usr/bin/env python3

import pickle
from bisect import bisect_left
from Bio import SeqIO
from subprocess import run
import sys
sys.path.append('../../utils/')
from get_mapping import get_mapping


if __name__ == '__main__':

    org_mapping,chr_mapping = get_mapping()

    coords = {}
    seqid = 1

    margin = int(sys.argv[1])
    genomes = sys.argv[2]

    to_write = []

    with open('../coords') as f:
        for line in f:
            line = line.strip().split()
            org,chromo,start,end,orientation,hit_name = line
            if org not in coords:
                coords[org] = {}
            if chromo not in coords[org]:
                coords[org][chromo] = []
            genome = SeqIO.parse(f'{genomes}/{org}.fasta','fasta')
            for seq in genome:
                if seq.id == chromo:
                    if orientation == 'forward':
                        seq = seq[max(int(start)-margin,0):min(int(end)+margin,len(seq))]
                        seq.id = f'{seqid} {org} {chromo} {start} {end}'
                        seq.description = ''
                        to_write.append(seq)
                    else:
                        seq = seq[max(int(start)-margin,0):min(int(end)+margin,len(seq))].reverse_complement()
                        seq.id = f'{seqid} {org} {chromo} {start} {end}'
                        seq.description = ''
                        to_write.append(seq)
                    coords[org][chromo].append((max(int(start)-margin,0),min(int(end)+margin,len(seq)),orientation,seqid))
                    break
            seqid += 1

    SeqIO.write(to_write,'to_align','fasta')

    lines = set()
    with open('dialign.homology') as f:
        for line in f:
            #input('next')
            name1,chromo1,start1,end1,hit_start1,hit_end1,name2,chromo2,start2,end2,hit_start2,hit_end2,score,length,flipped = line.strip().split()
            org1 = org_mapping[name1.split('ele')[0].split('chr')[0]]
            org2 = org_mapping[name2.split('ele')[0].split('chr')[0]]
            chromo1 = chr_mapping[org1][chromo1]
            chromo2 = chr_mapping[org2][chromo2]
            start1 = int(start1)
            start2 = int(start2)
            end1 = int(end1)
            end2 = int(end2)
            length = int(length)
            hit_start1 = int(hit_start1)
            hit_start2 = int(hit_start2)
            hit_end1 = int(hit_end1)
            hit_end2 = int(hit_end2)
            real_start1 = start1 + hit_start1
            real_end1 = real_start1 + length
            real_start2 = start2 + hit_start2
            real_end2 = real_start2 + length

            if org1 not in coords or chromo1 not in coords[org1] or org2 not in coords or chromo2 not in coords[org2]:
                continue
            for coord in coords[org1][chromo1]:
                start_coord1 = int(coord[0])
                end_coord1 = int(coord[1])
                if (real_start1 > start_coord1 and real_start1 < end_coord1) and (real_end1 > start_coord1 and real_end1 < end_coord1):
                    pass
                else:
                    continue
                ori1 = coord[2]
                seqid1 = coord[3]
                for coord2 in coords[org2][chromo2]:
                    start_coord2 = int(coord2[0])
                    end_coord2 = int(coord2[1])
                    if (real_start2 > start_coord2 and real_start2 < end_coord2) and (real_end2 > start_coord2 and real_end2 < end_coord2):
                        pass
                    else:
                        continue
                    ori2 = coord2[2]
                    seqid2 = coord2[3]
                    if ori1 == ori2 and flipped == 'forward':
                        if ori1 == 'reverse':
                            start1 = end_coord1 - real_end1
                            start2 = end_coord2 - real_end2
                        else:
                            start1 = real_start1 - start_coord1
                            start2 = real_start2 - start_coord2
                    elif ori1 != ori2 and flipped == 'reverse':
                        if ori1 == 'reverse':
                            start1 = end_coord1 - real_end1
                            start2 = real_start2 - start_coord2
                        else:
                            start1 = real_start1 - start_coord1
                            start2 = end_coord2 - real_end2
                    else:
                        continue
                    lines.add(f'{seqid1}\t{seqid2}\t{start1}\t{start2}\t{length}\t{score}\n')

                
    with open('to_align.anc','w') as f:
        for line in lines:
            f.write(line)



#! /usr/bin/env python3

import pickle
import sys
import os
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np

def main():
    if len(sys.argv) < 2:
        print("Usage: python3 plot_ortho_stats.py <output_dir>")
        sys.exit(1)
    outd = sys.argv[1]
    mfile = f'{outd}/master_orthology_edge_graph.pickle'
    if not os.path.exists(mfile):
        print(f"ERROR: {mfile} not found")
        print("Run compute_ortho_stats.py first")
        sys.exit(1)
    print("Loading master orthology edge graph...")
    with open(mfile, 'rb') as f:
        data = pickle.load(f)
    mgraph = data['graph']
    mode = data['mode']
    score_type = data['score_type']
    thresh = data['threshold']
    print(f"Mode: {mode}, Score: {score_type}, Threshold: {thresh}")
    pdir = f'{outd}/orthology_plots'
    os.makedirs(pdir, exist_ok=True)
    def cnt_edges(attr_filter=None, rbh_only=False):
        cnt = 0
        for u, v, d in mgraph.edges(data=True):
            if rbh_only and not d['is_rbh']:
                continue
            if attr_filter is None or d.get(attr_filter, False):
                cnt += 1
        return cnt
    def cnt_ch_edges(attr_filter=None, rbh_only=False):
        cnt = 0
        for u, v, d in mgraph.edges(data=True):
            if not d.get('in_chains', False):
                continue
            if rbh_only and not d['is_rbh']:
                continue
            if attr_filter is None or d.get(attr_filter, False):
                cnt += 1
        return cnt
    if mode == 'all-vs-all':
        stg = ['Initial', 'In chains', 'After\npairwise\ncograph', 'After\nalignment', 'Final']
        acnt = [cnt_edges(), cnt_edges('in_chains'), cnt_edges('after_pairwise_cograph'), cnt_edges('after_alignment'), cnt_edges('after_union_no_new_edges')]
        rcnt = [cnt_edges(rbh_only=True), cnt_edges('in_chains', rbh_only=True), cnt_edges('after_pairwise_cograph', rbh_only=True), cnt_edges('after_alignment', rbh_only=True), cnt_edges('after_union_no_new_edges', rbh_only=True)]
        fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(14, 6))
        ax1.bar(stg, acnt, alpha=0.7, edgecolor='black')
        ax1.set_ylabel('Number of edges')
        ax1.set_title(f'All-vs-all: All edges (>{thresh})')
        ax1.grid(axis='y', alpha=0.3)
        for i, v in enumerate(acnt):
            ax1.text(i, v, str(v), ha='center', va='bottom')
        ax2.bar(stg, rcnt, alpha=0.7, edgecolor='black', color='orange')
        ax2.set_ylabel('Number of edges')
        ax2.set_title('All-vs-all: RBH only')
        ax2.grid(axis='y', alpha=0.3)
        for i, v in enumerate(rcnt):
            ax2.text(i, v, str(v), ha='center', va='bottom')
        plt.tight_layout()
        plt.savefig(f'{pdir}/all_vs_all_pipeline.svg', format='svg', bbox_inches='tight')
        plt.savefig(f'{pdir}/all_vs_all_pipeline.pdf', format='pdf', bbox_inches='tight')
        plt.close()
        print(f"  Saved all-vs-all plot")
        chstg = ['Initial\n(in chains)', 'After\npairwise\ncograph', 'After\nalignment', 'Final']
        chacnt = [cnt_ch_edges(), cnt_ch_edges('after_pairwise_cograph'), cnt_ch_edges('after_alignment'), cnt_ch_edges('after_union_no_new_edges')]
        chrcnt = [cnt_ch_edges(rbh_only=True), cnt_ch_edges('after_pairwise_cograph', rbh_only=True), cnt_ch_edges('after_alignment', rbh_only=True), cnt_ch_edges('after_union_no_new_edges', rbh_only=True)]
        fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(14, 6))
        ax1.bar(chstg, chacnt, alpha=0.7, edgecolor='black')
        ax1.set_ylabel('Number of edges')
        ax1.set_title(f'Chains-only: All edges (>{thresh})')
        ax1.grid(axis='y', alpha=0.3)
        for i, v in enumerate(chacnt):
            ax1.text(i, v, str(v), ha='center', va='bottom')
        ax2.bar(chstg, chrcnt, alpha=0.7, edgecolor='black', color='orange')
        ax2.set_ylabel('Number of edges')
        ax2.set_title('Chains-only: RBH only')
        ax2.grid(axis='y', alpha=0.3)
        for i, v in enumerate(chrcnt):
            ax2.text(i, v, str(v), ha='center', va='bottom')
        plt.tight_layout()
        plt.savefig(f'{pdir}/chains_only_pipeline.svg', format='svg', bbox_inches='tight')
        plt.savefig(f'{pdir}/chains_only_pipeline.pdf', format='pdf', bbox_inches='tight')
        plt.close()
        print(f"  Saved chains-only plot")
    else:
        stg = ['Initial', 'After\npairwise\ncograph', 'After\nalignment', 'Final']
        acnt = [cnt_edges(), cnt_edges('after_pairwise_cograph'), cnt_edges('after_alignment'), cnt_edges('after_union_no_new_edges')]
        rcnt = [cnt_edges(rbh_only=True), cnt_edges('after_pairwise_cograph', rbh_only=True), cnt_edges('after_alignment', rbh_only=True), cnt_edges('after_union_no_new_edges', rbh_only=True)]
        fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(14, 6))
        ax1.bar(stg, acnt, alpha=0.7, edgecolor='black')
        ax1.set_ylabel('Number of edges')
        ax1.set_title(f'Chains-only: All edges (>{thresh})')
        ax1.grid(axis='y', alpha=0.3)
        for i, v in enumerate(acnt):
            ax1.text(i, v, str(v), ha='center', va='bottom')
        ax2.bar(stg, rcnt, alpha=0.7, edgecolor='black', color='orange')
        ax2.set_ylabel('Number of edges')
        ax2.set_title('Chains-only: RBH only')
        ax2.grid(axis='y', alpha=0.3)
        for i, v in enumerate(rcnt):
            ax2.text(i, v, str(v), ha='center', va='bottom')
        plt.tight_layout()
        plt.savefig(f'{pdir}/chains_only_pipeline.svg', format='svg', bbox_inches='tight')
        plt.savefig(f'{pdir}/chains_only_pipeline.pdf', format='pdf', bbox_inches='tight')
        plt.close()
        print(f"  Saved chains-only plot")
    if 'per_species_stats' in data:
        print("\nGenerating per-species plots...")
        pss = data['per_species_stats']
        splist = pss['species_list']
        if mode == 'chains':
            stg = ['Initial', 'After\nPCG', 'After\nAlignment', 'Final']
            skeys = ['initial', 'after_pairwise_cograph', 'after_alignment', 'after_union_no_new_edges']
            x = np.arange(len(stg))
            width = 0.08
            nspe = len(splist)
            fig, ax = plt.subplots(figsize=(14, 8))
            colors = plt.cm.tab10(np.linspace(0, 1, nspe))
            for i, sp in enumerate(splist):
                cnts = [pss[k].get(sp, 0) for k in skeys]
                offset = (i - nspe / 2) * width
                ax.bar(x + offset, cnts, width, label=sp, color=colors[i], alpha=0.8, edgecolor='black', linewidth=0.5)
            ax.set_ylabel('Number of edges', fontsize=12)
            ax.set_title('Edge counts per species through pipeline stages', fontsize=14)
            ax.set_xticks(x)
            ax.set_xticklabels(stg)
            ax.legend(loc='upper right', fontsize=9, ncol=2)
            ax.grid(axis='y', alpha=0.3)
            plt.tight_layout()
            plt.savefig(f'{pdir}/per_species_pipeline.svg', format='svg', bbox_inches='tight')
            plt.savefig(f'{pdir}/per_species_pipeline.pdf', format='pdf', bbox_inches='tight')
            plt.close()
            print(f"  Saved per-species plot")
            fig, ax = plt.subplots(figsize=(10, 6))
            for i, sp in enumerate(splist):
                cnts = [pss[k].get(sp, 0) for k in skeys]
                ax.plot(stg, cnts, marker='o', label=sp, linewidth=2, markersize=6)
            ax.set_ylabel('Number of edges', fontsize=12)
            ax.set_xlabel('Pipeline stage', fontsize=12)
            ax.set_title('Edge counts per species through pipeline stages', fontsize=14)
            ax.legend(loc='upper right', fontsize=9, ncol=2)
            ax.grid(alpha=0.3)
            plt.tight_layout()
            plt.savefig(f'{pdir}/per_species_pipeline_lines.svg', format='svg', bbox_inches='tight')
            plt.savefig(f'{pdir}/per_species_pipeline_lines.pdf', format='pdf', bbox_inches='tight')
            plt.close()
            print(f"  Saved per-species line plot")
            if 'rbh' in pss:
                fig, axes = plt.subplots(2, 5, figsize=(20, 8))
                axes = axes.flatten()
                for i, sp in enumerate(splist):
                    acnt = [pss[k].get(sp, 0) for k in skeys]
                    rcnt = [pss['rbh'][k].get(sp, 0) for k in skeys]
                    xpos = np.arange(len(stg))
                    w = 0.35
                    axes[i].bar(xpos - w/2, acnt, w, label='All edges', alpha=0.8, edgecolor='black')
                    axes[i].bar(xpos + w/2, rcnt, w, label='RBH only', alpha=0.8, edgecolor='black', color='orange')
                    axes[i].set_title(sp.replace('GCF_', ''), fontsize=10)
                    axes[i].set_xticks(xpos)
                    axes[i].set_xticklabels(stg, fontsize=8, rotation=45, ha='right')
                    axes[i].legend(fontsize=8)
                    axes[i].grid(axis='y', alpha=0.3)
                plt.suptitle('Per-species edge counts: All vs RBH', fontsize=14)
                plt.tight_layout()
                plt.savefig(f'{pdir}/per_species_all_vs_rbh.svg', format='svg', bbox_inches='tight')
                plt.savefig(f'{pdir}/per_species_all_vs_rbh.pdf', format='pdf', bbox_inches='tight')
                plt.close()
                print(f"  Saved per-species All vs RBH comparison plot")
        else:
            stg = ['Initial', 'In\nChains', 'After\nPCG', 'After\nAlignment', 'Final']
            skeys = ['initial', 'in_chains', 'after_pairwise_cograph', 'after_alignment', 'after_union_no_new_edges']
            x = np.arange(len(stg))
            width = 0.08
            nspe = len(splist)
            fig, ax = plt.subplots(figsize=(16, 8))
            colors = plt.cm.tab10(np.linspace(0, 1, nspe))
            for i, sp in enumerate(splist):
                cnts = [pss[k].get(sp, 0) for k in skeys]
                offset = (i - nspe / 2) * width
                ax.bar(x + offset, cnts, width, label=sp, color=colors[i], alpha=0.8, edgecolor='black', linewidth=0.5)
            ax.set_ylabel('Number of edges', fontsize=12)
            ax.set_title('Edge counts per species (all-vs-all mode)', fontsize=14)
            ax.set_xticks(x)
            ax.set_xticklabels(stg)
            ax.legend(loc='upper right', fontsize=9, ncol=2)
            ax.grid(axis='y', alpha=0.3)
            plt.tight_layout()
            plt.savefig(f'{pdir}/per_species_all_vs_all.svg', format='svg', bbox_inches='tight')
            plt.savefig(f'{pdir}/per_species_all_vs_all.pdf', format='pdf', bbox_inches='tight')
            plt.close()
            print(f"  Saved per-species all-vs-all plot")
            if 'chains_only' in pss:
                stg = ['Initial\n(in chains)', 'After\nPCG', 'After\nAlignment', 'Final']
                skeys = ['initial', 'after_pairwise_cograph', 'after_alignment', 'after_union_no_new_edges']
                x = np.arange(len(stg))
                fig, ax = plt.subplots(figsize=(14, 8))
                for i, sp in enumerate(splist):
                    cnts = [pss['chains_only'][k].get(sp, 0) for k in skeys]
                    offset = (i - nspe / 2) * width
                    ax.bar(x + offset, cnts, width, label=sp, color=colors[i], alpha=0.8, edgecolor='black', linewidth=0.5)
                ax.set_ylabel('Number of edges', fontsize=12)
                ax.set_title('Edge counts per species (chains-only subset)', fontsize=14)
                ax.set_xticks(x)
                ax.set_xticklabels(stg)
                ax.legend(loc='upper right', fontsize=9, ncol=2)
                ax.grid(axis='y', alpha=0.3)
                plt.tight_layout()
                plt.savefig(f'{pdir}/per_species_chains_only.svg', format='svg', bbox_inches='tight')
                plt.savefig(f'{pdir}/per_species_chains_only.pdf', format='pdf', bbox_inches='tight')
                plt.close()
                print(f"  Saved per-species chains-only plot")
    print(f"\nPlots saved to {pdir}/")

if __name__ == '__main__':
    main()



#! /usr/bin/env python3

import pickle
from sys import argv
from pprint import pprint
from os.path import isfile

def check_orientation(match_bib):
    global cases
    global all_same
    oris = []
    if 'matches' in match_bib:
        for org,bib in match_bib['matches'].items():
            if bib['match is on other strand in other genome']:
                oris.append('reverse')
            else:
                oris.append('forward')
        if len(set(oris)) == 1:
            all_same += 1
        cases += 1
        f_count = oris.count('forward')
        r_count = oris.count('reverse')
    elif 'dups_matches' in match_bib:
        for org,bib in match_bib['dups_matches'].items():
            if org == 'syntenic':
                continue
            if bib['match is on other strand in other genome']:
                oris.append('reverse')
            else:
                oris.append('forward')
        if len(set(oris)) == 1:
            all_same += 1
        cases += 1
        f_count = oris.count('forward')
        r_count = oris.count('reverse')
    else:
        return(False)
    if f_count >= r_count:
        return('forward')
    else:
        return('reverse')
    return(False)

def check_forward_ambiguity(match_bib):
    if match_bib['meta']['multiple matches out of tolerance range'] == 1 or match_bib['meta']['matches have ambiguous matches (tolerance/chromosome out of range)'] == 1:
        return(True)
    return(False)

org = argv[1]
anchor_dir = argv[2]

new = {}
cases = 0
all_same = 0

if isfile(anchor_dir+f'/aligned/{org}_with_syn_eval'):
    ad = anchor_dir+f'/aligned/{org}_with_syn_eval'
else:
    ad = anchor_dir+f'/aligned/{org}'

with open(ad,'rb') as f:
    aligned = pickle.load(f)
for i,bib1 in aligned.items():
    valid = False
    new_entry = {'chromosome':bib1['chromosome'],'start':bib1['start'],'end':bib1['end'],'matches':{}}
    for org2,bib2 in bib1['matches'].items():
        # take out ones without matches 
        if 'matches' in bib2 and len(bib2['matches'])==0 and 'dups_matches' in bib2 and len(bib2['dups_matches'])==0:
            continue
        # take out ones with forward ambiguity
        if check_forward_ambiguity(bib2):
            continue
        valid = True
        # check orientation
        checked_orientation = check_orientation(bib2)
        new_entry['matches'][org2] = {}
        if 'matches' in bib2:
            for j,bib3 in sorted(bib2['matches'].items()):
                if bib3['match is on other strand in other genome']:
                    ori = 'reverse'
                else:
                    ori = 'forward'
                new_entry['matches'][org2][j] = (int(bib3['match score']),bib3[f'hit coordinates in (own) {org} candidate'],ori)
                break
        if 'dups_matches' in bib2 and 'syntenic' in bib2['dups_matches'] and len(new_entry['matches'][org2]) == 0:
            syn = bib2['dups_matches']['syntenic']
            del bib2['dups_matches']['syntenic']
            for j,bib3 in sorted(bib2['dups_matches'].items()):
                if j not in syn:
                    continue
                if bib3['match is on other strand in other genome']:
                    ori = 'reverse'
                else:
                    ori = 'forward'
                new_entry['matches'][org2][j] = (int(bib3['match score']),bib3[f'hit coordinates in (own) {org} candidate'],ori)
                break
    if valid:
        new[i] = new_entry

with open(f'compressed_maps_multis_to_one/{org}','wb') as f:
    pickle.dump(new,f)



#! /usr/bin/env python3

from sys import argv
from subprocess import run
import pickle
from os.path import isfile
import sys

def get_chr_l(org,chromo):
    idx = small_meta[org][0].index(chromo)
    if idx == 0:
        return(small_meta[org][1][0])
    else:
        return(small_meta[org][1][idx] - small_meta[org][1][idx-1])

ref_genome = argv[1]

small_meta_path = '../../utils/small_meta/'
anchors_path = '../../utils/anchors/aligned/'
anchors_path_cand = '../../utils/anchors/candidates/'
small_meta = {}

target_genome = argv[2]
orgs = [ref_genome,target_genome]
for org in orgs:
    with open(f'{small_meta_path}/{org}','rb') as f:
        small_meta[org] = pickle.load(f)

targets = [target_genome]
chromo_mapping2 = {}

orders_others = {}
oris_others = {}
orders_others[target_genome] = {}
oris_others[target_genome] = {}

with open(anchors_path+f'/{ref_genome}','rb') as f:
    anchors = pickle.load(f)
with open(anchors_path_cand+f'/{target_genome}','rb') as f:
    t_anchors = pickle.load(f)

mapping_contig_chromo = {}

first = int(argv[3])
if first == 1:
    iss = sorted(list(anchors.keys()))
else:
    iss = []
    ref_order = []
    with open(f'orders/{ref_genome}') as f:
        for line in f:
            seq, ori = line.strip().split('\t')
            ref_order.append((seq,ori))
    for seq,ori in ref_order:
        tmp = []
        for i,bib in anchors.items():
            if bib['chromosome'] == seq:
                tmp.append(i)
        iss += sorted(tmp)

for i in iss:
    chromo = anchors[i]['chromosome'] 
    if get_chr_l(ref_genome,chromo) < int(argv[4]):
        continue
    if chromo not in orders_others[target_genome]:
        orders_others[target_genome][chromo] = []
        oris_others[target_genome][chromo] = {}
    if 'matches' in anchors[i] and target_genome in anchors[i]['matches'] and 'matches' in anchors[i]['matches'][target_genome] and anchors[i]['matches'][target_genome]['meta']['multiple matches out of tolerance range'] == 0:
        for j,match_bib in anchors[i]['matches'][target_genome]['matches'].items():
            chromo2 = t_anchors[j]['chromosome'] 
            if get_chr_l(target_genome,chromo2) < int(argv[4]):
                continue
            if chromo2 not in oris_others[target_genome][chromo]:
                oris_others[target_genome][chromo][chromo2] = []
            if chromo2 not in mapping_contig_chromo:
                mapping_contig_chromo[chromo2] = {}
            if chromo not in mapping_contig_chromo[chromo2]:
                mapping_contig_chromo[chromo2][chromo] = 0
            mapping_contig_chromo[chromo2][chromo] += 1
            oris = match_bib['match is on other strand in other genome']
            orders_others[target_genome][chromo].append(chromo2)
            if oris==0:
                oris_others[target_genome][chromo][chromo2].append('forward')
            else:
                oris_others[target_genome][chromo][chromo2].append('reverse')

f2 = open(f'singles_out/significantly_divergent_orientations_from_AncST_target_{target_genome}_ref_{ref_genome}.txt','w')
orientations = {}
with open(f'singles_out/orientations_and_hits_from_AncST_target_{target_genome}_ref_{ref_genome}.txt','w') as f:
    for org in oris_others:
        for chromo in oris_others[org]:
            for chromo2 in oris_others[org][chromo]:
                if oris_others[org][chromo][chromo2].count('forward') > oris_others[org][chromo][chromo2].count('reverse'):
                    f.write(f'{chromo2} - {chromo} - forward ({oris_others[org][chromo][chromo2].count("forward")} out of {len(oris_others[org][chromo][chromo2])})\n')
                    if chromo2 not in orientations:
                        orientations[chromo2] = {}
                    orientations[chromo2][chromo] = '+'
                else:
                    f.write(f'{chromo2} - {chromo} - reverse ({oris_others[org][chromo][chromo2].count("reverse")} out of {len(oris_others[org][chromo][chromo2])})\n')
                    if chromo2 not in orientations:
                        orientations[chromo2] = {}
                    orientations[chromo2][chromo] = '-'
                tot_counts = len(oris_others[org][chromo][chromo2])
                if oris_others[org][chromo][chromo2].count('forward') > tot_counts*0.1 and oris_others[org][chromo][chromo2].count('forward') < tot_counts*0.9:
                    f2.write(f'{chromo2} - {chromo} - for:  {oris_others[org][chromo][chromo2].count("forward")} , rev: {oris_others[org][chromo][chromo2].count("reverse")}\n')
f2.close()

if len(orders_others) == 0:
    print(f'no syn between {ref_genome} and {targets[0]}')
    exit(0)

simple_map = {}
for t_chromo,bib in mapping_contig_chromo.items():
    r_chromos_with_count = list(bib.items())
    simple_map[t_chromo] = r_chromos_with_count[[x[1] for x in r_chromos_with_count].index(max([x[1] for x in r_chromos_with_count]))]

g_taken = {}
written = {}
new_orders_others = {targets[0]:{}}
contig_appearance_counter = {}

for ref in small_meta[ref_genome][0]:
    if ref not in orders_others[targets[0]]:
        #print(f'{ref_genome} has chromo {ref} which doesnt have anchor alignments to any target chromsomes')
        continue
    new_orders_others[targets[0]][ref] = []
    c = orders_others[targets[0]][ref]
    anything_valid = 0
    new_scaffold_len = 0
    taken = []
    for x in c:
        if x not in contig_appearance_counter:
            contig_appearance_counter[x] = 0
        contig_appearance_counter[x] += 1
        if x not in written and simple_map[x][0] == ref and contig_appearance_counter[x] >= simple_map[x][1]/2:
            anything_valid = 1
            written[x] = 1
            new_orders_others[targets[0]][ref].append(x)
            taken.append(x)
    if anything_valid == 1:
        for x in taken:
            g_taken[x] = 1
               
orders_others = new_orders_others

with open(f'simple_maps/{target_genome}','wb') as f:
    pickle.dump(simple_map,f)

### make main output file with contig order
written = {}
with open(f'orders/{target_genome}','w') as f:
    ref_order = []
    if first == 1:
        ref_order = [(x,'') for x in small_meta[ref_genome][0]]
    else:
        with open(f'orders/{ref_genome}') as f2:
            for line in f2:
                seq, ori = line.strip().split('\t')
                ref_order.append((seq,ori))
    for ref,ori1 in ref_order:
        if ref not in new_orders_others[targets[0]]:
            continue
        c = new_orders_others[targets[0]][ref]
        if len(c) == 0:
            continue
        for x in c:
            if x not in written and simple_map[x][0] == ref:
                ori = orientations[x][ref]
                f.write(f'{x}\t{ori}\n')
                written[x] = 1
